/*******************************************************************************
*
*  FREESCALE SEMICONDUCTOR INC. 2004
*  ALL RIGHTS RESERVED
*  COPYRIGHT (c)
*  
********************************************************************************
*
*  DESCRIPTION:
*   The Motor-Control PWM - Commutation Capable (PWMC) function drives
*   a PWM phase with the feature to commutate (switch ON/OFF, change  
*   polarity, swap-channels).                                         
*   This function is not functional standalone. The PWMM (PWM Master) 
*   function must update the PWMC edge times.                         
*
********************************************************************************
*
*  REVISION HISTORY:
*                                                         
*  REVISION    AUTHOR       DATE        DESCRIPTION OF CHANGE                  
*  --------    -----------  ---------   ------------------------------------                  
*   1.0        M. Brejl     20/Sep/04   Ready for set3 release.
*
*******************************************************************************/

#ifndef _ETPUC_PWMC__H_
#define _ETPUC_PWMC__H_

// commutation command structure
typedef union = {
  int32 cmd;
  struct {
    int8  base_ch_num:8;         // bits  7:0
    int8  base_ch_comm_byte:8;   // bits 15:8
    int8  compl_ch_comm_byte:8;  // bits 23:16
    int8  phase_options:8;       // bits 31:24
  } b;
} comm_cmd_t;

// Commutation command global variable
comm_cmd_t comm_cmd;

// commutation byte options
#define PWMC_COMM_DO_NOTHING       0x0
#define PWMC_COMM_OFF_LOW          0x1
#define PWMC_COMM_OFF_HIGH         0x2
#define PWMC_COMM_ON_ACTIVE_HIGH   0x3
#define PWMC_COMM_ON_ACTIVE_LOW    0x4

// function prototype
//void Commutate( comm_cmd_t comm_cmd );
//void Commutate( registerp comm_cmd );
void Commutate( void);  // 32-bit comm_cmd is passed through global variable


#endif  /* _ETPUC_PWMC__H_ */

/*********************************************************************
 *
 * Copyright:
 *	Freescale Semiconductor, INC. All Rights Reserved.
 *  You are hereby granted a copyright license to use, modify, and
 *  distribute the SOFTWARE so long as this entire notice is
 *  retained without alteration in any modified and/or redistributed
 *  versions, and that such modified versions are clearly identified
 *  as such. No licenses are granted by implication, estoppel or
 *  otherwise under any patents or trademarks of Freescale
 *  Semiconductor, Inc. This software is provided on an "AS IS"
 *  basis and without warranty.
 *
 *  To the maximum extent permitted by applicable law, Freescale
 *  Semiconductor DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED,
 *  INCLUDING IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 *  PARTICULAR PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT WITH
 *  REGARD TO THE SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF)
 *  AND ANY ACCOMPANYING WRITTEN MATERIALS.
 *
 *  To the maximum extent permitted by applicable law, IN NO EVENT
 *  SHALL Freescale Semiconductor BE LIABLE FOR ANY DAMAGES WHATSOEVER
 *  (INCLUDING WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
 *  BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER
 *  PECUNIARY LOSS) ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.
 *
 *  Freescale Semiconductor assumes no responsibility for the
 *  maintenance and support of this software
 ********************************************************************/

