/*******************************************************************************
*
*  FREESCALE SEMICONDUCTOR INC. 2004
*  ALL RIGHTS RESERVED
*  COPYRIGHT (c)
*  
********************************************************************************
*
*  DESCRIPTION:
*   ETPU Motor Control Function Set 3
*
********************************************************************************
*
*  REVISION HISTORY:
*                                                         
*  REVISION    AUTHOR        DATE         DESCRIPTION OF CHANGE                  
*  --------    -----------   ---------    ------------------------------------              
*  0.5         Milan Brejl   26/Oct/04    Updated for set3 release 0.5.
*              Michal Princ
*  
*******************************************************************************/ 

#include "etpuc_set3.h"

/**********************/
/* Define memory size */
/**********************/
#pragma memory ROM[MEMORY_SIZE - ((LAST_FUNCTION+1)*0x40) - 4] @ ((LAST_FUNCTION+1)*0x40);
#pragma memory LOCAL[0] @ 0x00;

/*******************************/
/* Define entry table location */
/*******************************/
/* Put entry table at the beginning of memory so that it is not
   dependent on memory size. */
#pragma ENTRYADDR 0x0000;

int24 Global_Error = 0;

void Global_Error_Func();

/*******************************************************************************
    SET1 - GENERAL PURPOSE FUNCTIONS
 ******************************************************************************/

/* PWM (Pulse Width Modulation) function */
#ifdef PWM_FUNCTION_NUMBER
    #include "etpuc_pwm.c"
#endif

/* IC (Input Capture) function */
#ifdef IC_FUNCTION_NUMBER
    #include "etpuc_ic.c"
#endif

/* OC (Output Compare) function */
#ifdef OC_FUNCTION_NUMBER
    #include "etpuc_oc.c"
#endif

/* PFM (Period and Frequency Measurement) function */
#ifdef PFM_FUNCTION_NUMBER
    #include "etpuc_pfm.c"
#endif

/* PPA (Period / Period Accumulation) function */
#ifdef PPA_FUNCTION_NUMBER
    #include "etpuc_ppa.c"
#endif

/* SM (Stepper Motor) function */
#ifdef SM_FUNCTION_NUMBER
    #include "etpuc_sm.c"
#endif

/* QOM (Queued Output Match) function */
#ifdef QOM_FUNCTION_NUMBER
    #include "etpuc_qom.c"
#endif

/* UART (Universal Asynchronous Recieve / Transmitter) function */
#ifdef UART_FUNCTION_NUMBER
    #include "etpuc_uart.c"
#endif

/* SPI (Synchronous Peripheral Interface) function */
#ifdef SPI_FUNCTION_NUMBER
  #include "etpuc_spi.c"
#endif

/* POC (Protected Output Compare) function */
#ifdef POC_FUNCTION_NUMBER
    #include "etpuc_icoc.c"
#endif

/* GPIO (General Purpose Input / Output) function */
#ifdef GPIO_FUNCTION_NUMBER
    #include "etpuc_gpio.c"
#endif


/*******************************************************************************
    SET2 - AUTOMOTIVE FUNCTIONS
 ******************************************************************************/




/*******************************************************************************
    SET3 - MOTOR CONTROL FUNCTIONS  (DC MOTORS)
 ******************************************************************************/

#include "etpuc_mc.h"

/* PWMM (PWM Master) function           */
#ifdef PWMM_FUNCTION_NUMBER
    #include "etpuc_pwmm.h"
    #include "etpuc_pwmm.c"
#endif

/* PWMF (PWM - Full Range) function     */
#ifdef PWMF_FUNCTION_NUMBER
    #include "etpuc_pwmf.c"
#endif

/* PWMC (PWM - Commutation Capable) function    */
#ifdef PWMC_FUNCTION_NUMBER
    #include "etpuc_pwmc.h"
    #include "etpuc_pwmc.c"
#endif

/* HD (Hall Decoder) function           */
#ifdef HD_FUNCTION_NUMBER
    #include "etpuc_hd.c"
#endif

/* QD (Quadrature Decoder) function     */
#ifdef QD_FUNCTION_NUMBER
    #include "etpuc_qd.c"
#endif

/* ASENSE0 (Analog Sensing 0) function  */
#ifdef ASENSE_FUNCTION_NUMBER
    #include "etpuc_asense.c"
#endif

/* PID (PID Controller) function        */
#ifdef PID_FUNCTION_NUMBER
  #include "etpuc_pid.c"
#endif

/* SC (Speed Controller) function       */
#ifdef SC_FUNCTION_NUMBER
  #include "etpuc_sc.c"
#endif

/* BC (Break Controller) function       */
#ifdef BC_FUNCTION_NUMBER
  #include "etpuc_bc.c"
#endif

/*******************************************************************************
    SET4 - MOTOR CONTROL FUNCTIONS   (AC MOTORS)
*******************************************************************************/

/* ACIMVHZ (AC Induction Motor Volts per Herts control) function    */
#ifdef ACIMVHZ_FUNCTION_NUMBER
    #include "etpuc_acimvhz.c"
#endif

/* PMSMVC (Permanent Magnet Synchronous Motor Vector Control
            - Current Loop ) function                               */
#ifdef PMSMVC_FUNCTION_NUMBER
    #include "etpuc_pmsmvc.c"
#endif


/* ACIMVC (AC Induction Motor Vector Control
           Current Loop ) function                                  */
#ifdef ACIMVC_FUNCTION_NUMBER
    #include "etpuc_acimvc.c"
#endif

/*******************************************************************************
    TEST FUNCTIONS
*******************************************************************************/

/* TESTREG  */
#ifdef TEST_FUNCTION_NUMBER
    #include "etpuc_test.c"
#endif

/* IRQ  */
#ifdef IRQ_FUNCTION_NUMBER
    #include "etpuc_irq.c"
#endif




/*******************************************************************************
    GLOBAL ERROR ETPU FUNCTION
*******************************************************************************/

void Global_Error_Func()
{
	int24 error_temp;

	if (LinkServiceRequest == 1) error_temp=1;
	else
		error_temp = 0;

	if (MatchALatch == 1) error_temp++;
	error_temp <<= 1;

	if (MatchBLatch == 1) error_temp++;
	error_temp <<= 1;

	if (TransitionALatch == 1) error_temp++;
	error_temp <<= 1;

	if (TransitionBLatch == 1) error_temp++;
	error_temp <<= 8;

	Global_Error = error_temp + chan;

	ClearAllLatches();
}


/* output eTPU code image and information for CPU */
#pragma write h, (::ETPUfilename (cpu/etpu_set3.h));
#pragma write h, (/* Auto-generated file DO NOT EDIT */);
#pragma write h, ();
#pragma write h, (/* eTPU motor-control function set3 */ );
#pragma write h, (#define FS_ETPU_ENTRY_TABLE ::ETPUentrybase);
#pragma write h, ();
#pragma write h, (#define FS_ETPU_MISC ::ETPUmisc);
#pragma write h, ();
#pragma write h, (const uint32_t etpu_globals[] = { ::ETPUglobalimage }; );
#pragma write h, ();
#pragma write h, (const uint32_t etpu_code[] = { ::ETPUcode32 }; );
#pragma write h, ();
#pragma write h, (/* List of functions: );
#pragma write h, (::ETPUnames);
#pragma write h, (*/);
#pragma write h, ();
#pragma write h, (/* This function set presumes the following settings:);
#pragma write h, (#define ETPU_TCR1CTL_DIV2);
#pragma write h, (*/);


/*********************************************************************
 *
 * Copyright:
 *	Freescale Semiconductor, INC. All Rights Reserved.
 *  You are hereby granted a copyright license to use, modify, and
 *  distribute the SOFTWARE so long as this entire notice is
 *  retained without alteration in any modified and/or redistributed
 *  versions, and that such modified versions are clearly identified
 *  as such. No licenses are granted by implication, estoppel or
 *  otherwise under any patents or trademarks of Freescale
 *  Semiconductor, Inc. This software is provided on an "AS IS"
 *  basis and without warranty.
 *
 *  To the maximum extent permitted by applicable law, Freescale
 *  Semiconductor DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED,
 *  INCLUDING IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 *  PARTICULAR PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT WITH
 *  REGARD TO THE SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF)
 *  AND ANY ACCOMPANYING WRITTEN MATERIALS.
 *
 *  To the maximum extent permitted by applicable law, IN NO EVENT
 *  SHALL Freescale Semiconductor BE LIABLE FOR ANY DAMAGES WHATSOEVER
 *  (INCLUDING WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
 *  BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER
 *  PECUNIARY LOSS) ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.
 *
 *  Freescale Semiconductor assumes no responsibility for the
 *  maintenance and support of this software
 ********************************************************************/
